@@ -7,7 +7,7 @@
 //! implementations of the `crypto::Session` trait.
 
 use std::{
-    convert::{TryFrom, TryInto},
+    convert::TryFrom,
     net::{Ipv4Addr, Ipv6Addr, SocketAddrV4, SocketAddrV6},
 };
 
@@ -19,7 +19,6 @@ use crate::{
     cid_queue::CidQueue,
     coding::{BufExt, BufMutExt, UnexpectedEnd},
     config::{EndpointConfig, ServerConfig, TransportConfig},
-    crypto,
     shared::ConnectionId,
     ResetToken, Side, TransportError, VarInt, LOC_CID_COUNT, MAX_CID_SIZE, MAX_STREAM_COUNT,
     RESET_TOKEN_SIZE,
@@ -115,16 +114,13 @@ macro_rules! make_struct {
 apply_params!(make_struct);
 
 impl TransportParameters {
-    pub(crate) fn new<S>(
+    pub(crate) fn new(
         config: &TransportConfig,
-        endpoint_config: &EndpointConfig<S>,
+        endpoint_config: &EndpointConfig,
         cid_gen: &dyn ConnectionIdGenerator,
         initial_src_cid: ConnectionId,
-        server_config: Option<&ServerConfig<S>>,
-    ) -> Self
-    where
-        S: crypto::Session,
-    {
+        server_config: Option<&ServerConfig>,
+    ) -> Self {
         TransportParameters {
             initial_src_cid: Some(initial_src_cid),
             initial_max_streams_bidi: config.max_concurrent_bidi_streams,
@@ -134,12 +130,7 @@ impl TransportParameters {
             initial_max_stream_data_bidi_remote: config.stream_receive_window,
             initial_max_stream_data_uni: config.stream_receive_window,
             max_udp_payload_size: endpoint_config.max_udp_payload_size,
-            max_idle_timeout: config.max_idle_timeout.map_or(0u32.into(), |x| {
-                x.as_millis()
-                    .try_into()
-                    .expect("setter guarantees this is in-bounds")
-            }),
-            max_ack_delay: 0u32.into(),
+            max_idle_timeout: config.max_idle_timeout.unwrap_or(VarInt(0)),
             disable_active_migration: server_config.map_or(false, |c| !c.migration),
             active_connection_id_limit: if cid_gen.cid_len() == 0 {
                 2 // i.e. default, i.e. unsent
@@ -431,7 +422,7 @@ fn decode_cid(len: usize, value: &mut Option<ConnectionId>, r: &mut impl Buf) ->
         return Err(Error::Malformed);
     }
 
-    *value = Some(ConnectionId::new(&r.copy_to_bytes(len)));
+    *value = Some(ConnectionId::from_buf(r, len));
     Ok(())
 }
 
